// ganglia_metric.cc: Ganglia Metric Plugin
// Author: Eric Flumerfelt
// Last Modified: 11/14/2014
//
// An implementation of the MetricPlugin interface for Ganglia

#ifndef __GANGLIA_METRIC__
#define __GANGLIA_METRIC__ 1

#include "artdaq-utilities/Plugins/MetricMacros.hh"
#include "fhiclcpp/fwd.h"
#include "send_gmetric.h"

#include <boost/filesystem.hpp>
namespace bfs = boost::filesystem;

/**
 * \brief The artdaq namespace
 */
namespace artdaq {
/**
 * \brief An instance of the MetricPlugin class that sends metric data to Ganglia
 */
class GangliaMetric : public MetricPlugin {
 private:
  std::string configFile_;
  std::string group_;
  std::string cluster_;

 public:
  /**
   * \brief Construct an instance of the Ganglia metric
   * \param pset Parameter set with which to configure the MetricPlugin. Also includes "configFile" (path to
   * gmond.conf), "group" (Group for metrics) and "cluster" (Cluster tag to use (optional)). \param app_name Name of the
   * application sending metrics
   */
  explicit GangliaMetric(fhicl::ParameterSet const& pset, std::string const& app_name)
      : MetricPlugin(pset, app_name),
        configFile_(pset.get<std::string>("configFile", "")),
        group_(pset.get<std::string>("group", "ARTDAQ")),
        cluster_(pset.get<std::string>("cluster", "")) {
    if (configFile_ == "" || bfs::exists(configFile_)) {
      int sts = init_gmetric(configFile_.c_str());
      if (sts != 0) {
        throw cet::exception("GangliaMetric") << "Unable to configure Ganglia";
      }
    } else {
      throw cet::exception("GangliaMetric") << "Configuration file " << configFile_ << " does not exist!";
    }
  }

  virtual ~GangliaMetric() {
    MetricPlugin::stopMetrics();
    destroy_gmetric();
  }

  /**
   * \brief Gets the unique library name of this plugin
   * \return The library name of this plugin, "ganglia".
   */
  std::string getLibName() const override { return "ganglia"; }
  /**
   * \brief Ganglia does not need any specific action on stop
   */
  void stopMetrics_() override {}
  /**
   * \brief Ganglia does not need any specific action on start
   */
  void startMetrics_() override {}

  /**
   * \brief Send a string metric to Ganglia
   * \param name Name of the metric
   * \param value Value of the metric
   * \param unit Units of the metric
   */
  void sendMetric_(const std::string& name, const std::string& value, const std::string& unit) override {
    send_gmetric(name.c_str(), value.c_str(), "string", unit.c_str(), "both", 15, 0, group_.c_str(), cluster_.c_str(),
                 "", "");
  }

  /**
   * \brief Send a integer metric to Ganglia (truncated to int32)
   * \param name Name of the metric
   * \param value Value of the metric
   * \param unit Units of the metric
   */
  void sendMetric_(const std::string& name, const int& value, const std::string& unit) override {
    send_gmetric(name.c_str(), std::to_string(value).c_str(), "int32", unit.c_str(), "both", 15, 0, group_.c_str(),
                 cluster_.c_str(), "", "");
  }

  /**
   * \brief Send a double metric to Ganglia
   * \param name Name of the metric
   * \param value Value of the metric
   * \param unit Units of the metric
   */
  void sendMetric_(const std::string& name, const double& value, const std::string& unit) override {
    send_gmetric(name.c_str(), std::to_string(value).c_str(), "double", unit.c_str(), "both", 15, 0, group_.c_str(),
                 cluster_.c_str(), "", "");
  }

  /**
   * \brief Send a float metric to Ganglia
   * \param name Name of the metric
   * \param value Value of the metric
   * \param unit Units of the metric
   */
  void sendMetric_(const std::string& name, const float& value, const std::string& unit) override {
    send_gmetric(name.c_str(), std::to_string(value).c_str(), "float", unit.c_str(), "both", 15, 0, group_.c_str(),
                 cluster_.c_str(), "", "");
  }

  /**
   * \brief Send an unsigned long metric to Ganglia (truncated to uint32)
   * \param name Name of the metric
   * \param value Value of the metric
   * \param unit Units of the metric
   */
  void sendMetric_(const std::string& name, const unsigned long int& value, const std::string& unit) override {
    send_gmetric(name.c_str(), std::to_string(value).c_str(), "uint32", unit.c_str(), "both", 15, 0, group_.c_str(),
                 cluster_.c_str(), "", "");
  }
};
}  // End namespace artdaq

DEFINE_ARTDAQ_METRIC(artdaq::GangliaMetric)

#endif  // End ifndef __GANGLIA_METRIC__
